{
 TPopupbutton97 v1.0
  1997 Marc Nieuwhof, MNIware

 Draws a duo popupbutton with an image in the
 left pane and a down arrow in the right pane.
 On button with two panes that act as buttons themselves.
 These buttons first occured on the Office97 toolbars

 Properties:
 Glyph: TBitmap,
        A bitmap image for use in the left pane

 Events:
 OnLeftClick:  Triggered when the left pane was clicked
 OnRightClick: Triggered when the right pane was clicked

 There are no extra gadgets with this component, you
 must do the fancy stuff yourself in the OnLeftClick and
 OnRightClick events.

 For comments and/or suggestions you can write mail to:
 m.nieuwhof@hs-ipabo.edu

}
unit PopBut97;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  Menus;

   // Paintbutton states
Const
   pbDownAll     = 0;
   pbUpAll       = 1;
   pbDownLeft    = 2;
   pbDownRight   = 3;
   pbDownArrow   = 4;
   pbGlyphNormal = 5;
   pbGlyphDown   = 6;

Type
  TPopupButton97 = class(TCustomControl)
  private
    //
    //Private declarations
    //
    FGlyph:        TBitmap;
    FOnLeftClick:  TNotifyEvent;
    FOnRightClick: TNotifyEvent;
    // Internals
    HasMouse:    Boolean;
    StillDown:   Boolean;
    BigButton:   Boolean;
    // Some graphic areas
    Rc,
    RcL,
    RcR:         TRect;
    Bf:          Tfont;
    RightWidth:  Integer;
    // Internals
    Procedure SetGlyph(Value: TBitmap);
    Procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;  // New event
    Procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;  // New event
    Procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    Procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    Procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
  protected
    //
    // Protected Declarations
    //
    Procedure PaintButton(Mode: Integer);
    Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    //
    // Public declarations
    //
    Constructor Create(AOwner: TComponent); override;
    Destructor  Destroy; override;
    Procedure   Paint; Override;
    Procedure   Notification (AComponent: TComponent; Operation: TOperation); override;
  published
    //
    // Published declarations
    // New component specific properties
    //
    Property Glyph: TBitmap
             Read   FGlyph
             Write  SetGlyph;
    Property OnLeftClick: TNotifyEvent
             Read  FOnLeftClick
             Write FOnLeftClick;
    Property OnRightClick: TNotifyEvent
             Read  FOnRightClick
             Write FOnRightClick;
    // Inherited Properties
    Property Align;
    Property Enabled;
    Property Font;
    Property Hint;
    Property ShowHint;
    Property Left;
    Property Top;
    Property Width;
    Property Visible;
    Property Height;
    // Inherited Methods
    Property OnDragDrop;
    Property OnDragOver;
    Property OnEndDrag;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
    Property OnStartDrag;
  End;

procedure Register;

implementation
//
// Register component on palette
//
procedure Register;
begin
  RegisterComponents('Win32', [TPopupButton97]);
end;
//
// Create constructor
//
Constructor TPopupButton97.Create(AOwner: TComponent);
Begin
    Inherited Create(Aowner);
    ControlStyle := [csCaptureMouse,
                     csClickEvents,
                     csSetCaption,
                     csDoubleClicks];
    Width:=35;
    Height:=25;
    RightWidth:=11;
    Bf:=TFont.Create;       // Create font backup
    FGlyph:=TBitmap.Create; // Create Glyph bitmap
End;
//
// Destroy destructor
//
Destructor TPopupButton97.Destroy;
Begin
    Bf.Free;      // Free font backup
    FGlyph.Free;  // Free Glyph bitmap
    Inherited Destroy;
End;
//
// Paint method
//
Procedure TPopupButton97.Paint;
Begin
    Inherited paint;
    With Canvas do begin
        // Clear the canvas
        Brush.Color:=clBtnFace;
        Brush.Style:=bsSolid;
        Rc:=GetClientRect;
        FillRect(Rc);
    End;
    // Paint the button
    If (csDesigning in ComponentState) then Begin
        PaintButton(pbGlyphNormal);
        PaintButton(pbUpAll);
    End;
    If HasMouse and (not StillDown) then begin
        PaintButton(pbGlyphNormal);
        PaintButton(pbUpAll);
    End;
    If HasMouse and StillDown and BigButton then begin
        PaintButton(pbGlyphDown);
        PaintButton(pbDownLeft);
    End;
    If HasMouse and StillDown and (not BigButton) then Begin
        PaintButton(pbGlyphNormal);
        PaintButton(pbDownRight);
    End;
    If (Not HasMouse) and StillDown then begin
        PaintButton(pbGlyphNormal);
        PaintButton(pbUpAll);
    End;
    If Not HasMouse then
        PaintButton(pbGlyphNormal);
    // Downarrow
    PaintButton(pbDownArrow);

end;
//
// Paint the buttonborder
//
Procedure TPopupButton97.PaintButton(Mode: Integer);
Begin
    With Canvas do begin
        // Get the rect for the left pane
        RcL:=GetClientRect;
        SetRect(RcL,0,0,Width-RightWidth,Height);
        // Get the rect for the right pane
        RcR:=GetClientRect;
        SetRect(RcR,Width-RightWidth,0,Width,Height);
        // Draw the button in different states
        Case Mode of
        pbUpAll:
            Begin
            Frame3D(Canvas,RcL,clBtnHighlight,clBtnShadow,1);
            Frame3D(Canvas,RcR,clBtnHighlight,clBtnShadow,1);
            End;
        pbDownAll:
            Begin
            Frame3D(Canvas,RcL,clBtnShadow,clBtnHighLight,1);
            Frame3D(Canvas,RcR,clBtnShadow,clBtnHighlight,1);
            End;
        pbDownLeft:
            Begin
            Frame3D(Canvas,RcL,clBtnShadow,clBtnHighLight,1);
            Frame3D(Canvas,RcR,clBtnHighLight,clBtnShadow,1);
            End;
        pbDownRight:
            Begin
            Frame3D(Canvas,RcL,clBtnHighLight,clBtnShadow,1);
            Frame3D(Canvas,RcR,clBtnShadow,clBtnHighLight,1);
            End;
        pbDownArrow:
            Begin
            Brush.Style:=bsClear;
            Bf.Name:=Font.Name;
            Bf.Size:=Font.Size;
            Bf.CharSet:=Font.CharSet;
            Font.Name:='Marlett';
            Font.Size:=9;
            Font.Charset:=SYMBOL_CHARSET;
            TextOut(Width-RightWidth,(Height-TextHeight('0')) div 2,'6');
            Font.Name:=Bf.Name;
            Font.Size:=Bf.Size;
            Font.CharSet:=Bf.Charset;
            End;
        pbGlyphNormal:
            Begin
                If Assigned(FGlyph) then
                    Draw((RcL.Right-FGlyph.Width) div 2,(RcL.Bottom-FGlyph.Height) div 2,FGlyph);
            End;
        pbGlyphDown:
            Begin
                If Assigned(FGlyph) then
                    Draw(((RcL.Right-FGlyph.Width) div 2)+1,((RcL.Bottom-FGlyph.Height) div 2)+1,FGlyph);
            End;
        End;
    End;
End;
//
// Mouse detection
//
procedure TPopupButton97.CMMouseEnter(var Message: TMessage);
Begin
    If Enabled then begin
        HasMouse:=True;
        Invalidate;
    End;
End;
procedure TPopupButton97.CMMouseLeave(var Message: TMessage);
Begin
    If Enabled then begin
        HasMouse:=False;
        Invalidate;
    End;
End;
//
// Mouse down/up procedures
//
Procedure TPopupButton97.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin
    If Enabled then begin
        Inherited MouseDown(Button, Shift, X, Y);
        StillDown:=True;
        If X<Width-RightWidth then
            BigButton:=True
        Else
            BigButton:=False;
        Invalidate;
    End;
End;
Procedure TPopupButton97.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin
    If Enabled then begin
        Inherited MouseUp(Button, Shift, X, Y);
        StillDown:=False;
        Invalidate;
        If Assigned(FOnLeftClick) and BigButton and HasMouse then
            FOnLeftClick(Self);
        If Assigned(FOnRightClick) and (not BigButton) and HasMouse then
            FOnRightClick(Self);
    End;
End;
//
// IDE Designtime relevant procedures
//
Procedure TPopupButton97.CMFontChanged(var Msg: TMessage);
Begin
    Inherited;
    Canvas.Font:=Font;
    Invalidate;
End;
Procedure TPopupButton97.CMEnabledChanged(var Msg: TMessage);
Begin
    Inherited;
    Invalidate;
End;
Procedure TPopupButton97.CMTextChanged(var Msg: TMessage);
Begin
    Inherited;
    Invalidate;
End;
//
// Glyph property
//
Procedure TPopupButton97.SetGlyph(Value: TBitmap);
Begin
    FGlyph.Assign(Value);
    Invalidate;
End;
//
// Notification procedure for future use ;-)
//
Procedure TPopupButton97.Notification (AComponent: TComponent; Operation: TOperation);
Begin
    Inherited Notification(AComponent, Operation);
End;
end.
